# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
#
# Copyright (C) 2022, 2023 University Münster.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the Revised BSD License; see LICENSE file for
# more details.

"""Python API client wrapper for the DNB URN service API."""

from setuptools import setup

setup()
